@@ -16,7 +16,7 @@
         MA 02110-1301, USA.
 
 
-        xfwm4    - (c) 2002-2011 Olivier Fourdan
+        xfwm4    - (c) 2002-2015 Olivier Fourdan
 
  */
 
@@ -24,6 +24,7 @@
 #include "config.h"
 #endif
 
+#include <stdio.h>
 #include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -45,7 +46,7 @@
 #include "compositor.h"
 
 #ifndef MAX_HOSTNAME_LENGTH
-#define MAX_HOSTNAME_LENGTH 32
+#define MAX_HOSTNAME_LENGTH 512
 #endif /* MAX_HOSTNAME_LENGTH */
 
 #ifndef CURSOR_ROOT
@@ -63,8 +64,8 @@ handleXError (Display * dpy, XErrorEvent * err)
     char buf[64];
 
     XGetErrorText (dpy, err->error_code, buf, 63);
-    g_print ("XError: %s\n", buf);
-    g_print ("==>  XID 0x%lx, Request %d, Error %d <==\n",
+    fprintf (stderr, "XError: %s\n", buf);
+    fprintf (stderr, "==>  XID 0x%lx, Request %d, Error %d <==\n",
               err->resourceid, err->request_code, err->error_code);
 #endif
     return 0;
@@ -75,7 +76,8 @@ myDisplayInitAtoms (DisplayInfo *display_info)
 {
     static const char *atom_names[] = {
         "COMPOSITING_MANAGER",
-        "GNOME_PANEL_DESKTOP_AREA",
+        "_GTK_FRAME_EXTENTS",
+        "_GTK_HIDE_TITLEBAR_WHEN_MAXIMIZED",
         "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR",
         "KWM_WIN_ICON",
         "_MOTIF_WM_HINTS",
@@ -127,6 +129,7 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "_NET_WM_STATE_ABOVE",
         "_NET_WM_STATE_BELOW",
         "_NET_WM_STATE_DEMANDS_ATTENTION",
+        "_NET_WM_STATE_FOCUSED",
         "_NET_WM_STATE_FULLSCREEN",
         "_NET_WM_STATE_HIDDEN",
         "_NET_WM_STATE_MAXIMIZED_HORZ",
@@ -156,15 +159,6 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "PIXMAP",
         "SM_CLIENT_ID",
         "UTF8_STRING",
-        "_WIN_CLIENT_LIST",
-        "_WIN_DESKTOP_BUTTON_PROXY",
-        "_WIN_HINTS",
-        "_WIN_LAYER",
-        "_WIN_PROTOCOLS",
-        "_WIN_STATE",
-        "_WIN_SUPPORTING_WM_CHECK",
-        "_WIN_WORKSPACE",
-        "_WIN_WORKSPACE_COUNT",
         "WM_CHANGE_STATE",
         "WM_CLIENT_LEADER",
         "WM_CLIENT_MACHINE",
@@ -208,6 +202,7 @@ myDisplayInit (GdkDisplay *gdisplay)
     DisplayInfo *display;
     int major, minor;
     int dummy;
+    gchar *hostnametmp;
 
     display = g_new0 (DisplayInfo, 1);
 
@@ -310,28 +305,7 @@ myDisplayInit (GdkDisplay *gdisplay)
     display->have_xrandr = FALSE;
 #endif /* HAVE_RANDR */
 
-    display->root_cursor =
-        XCreateFontCursor (display->dpy, CURSOR_ROOT);
-    display->move_cursor =
-        XCreateFontCursor (display->dpy, CURSOR_MOVE);
-    display->busy_cursor =
-        cursorCreateSpinning (display->dpy);
-    display->resize_cursor[CORNER_TOP_LEFT] =
-        XCreateFontCursor (display->dpy, XC_top_left_corner);
-    display->resize_cursor[CORNER_TOP_RIGHT] =
-        XCreateFontCursor (display->dpy, XC_top_right_corner);
-    display->resize_cursor[CORNER_BOTTOM_LEFT] =
-        XCreateFontCursor (display->dpy, XC_bottom_left_corner);
-    display->resize_cursor[CORNER_BOTTOM_RIGHT] =
-        XCreateFontCursor (display->dpy, XC_bottom_right_corner);
-    display->resize_cursor[CORNER_COUNT + SIDE_LEFT] =
-        XCreateFontCursor (display->dpy, XC_left_side);
-    display->resize_cursor[CORNER_COUNT + SIDE_RIGHT] =
-        XCreateFontCursor (display->dpy, XC_right_side);
-    display->resize_cursor[CORNER_COUNT + SIDE_TOP] =
-        XCreateFontCursor (display->dpy, XC_top_side);
-    display->resize_cursor[CORNER_COUNT + SIDE_BOTTOM] =
-        XCreateFontCursor (display->dpy, XC_bottom_side);
+    myDisplayCreateCursor (display);
 
     myDisplayCreateTimestampWin (display);
 
@@ -344,12 +318,16 @@ myDisplayInit (GdkDisplay *gdisplay)
     display->nb_screens = 0;
     display->current_time = CurrentTime;
 
-    display->hostname = g_new0 (gchar, (size_t) MAX_HOSTNAME_LENGTH);
-    if (gethostname ((char *) display->hostname, MAX_HOSTNAME_LENGTH - 1))
+    hostnametmp = g_new0 (gchar, (size_t) MAX_HOSTNAME_LENGTH + 1);
+    if (gethostname ((char *) hostnametmp, MAX_HOSTNAME_LENGTH))
     {
-        g_free (display->hostname);
+        g_warning ("The display's hostname could not be determined.");
         display->hostname = NULL;
+    } else {
+        hostnametmp[MAX_HOSTNAME_LENGTH] = '\0';
+        display->hostname = g_strdup(hostnametmp);
     }
+    g_free (hostnametmp);
 
     compositorInitDisplay (display);
 
@@ -359,14 +337,7 @@ myDisplayInit (GdkDisplay *gdisplay)
 DisplayInfo *
 myDisplayClose (DisplayInfo *display)
 {
-    int i;
-
-    XFreeCursor (display->dpy, display->busy_cursor);
-    display->busy_cursor = None;
-    XFreeCursor (display->dpy, display->move_cursor);
-    display->move_cursor = None;
-    XFreeCursor (display->dpy, display->root_cursor);
-    display->root_cursor = None;
+    myDisplayFreeCursor (display);
     XDestroyWindow (display->dpy, display->timestamp_win);
     display->timestamp_win = None;
 
@@ -376,12 +347,6 @@ myDisplayClose (DisplayInfo *display)
         display->hostname = NULL;
     }
 
-    for (i = 0; i < SIDE_COUNT + CORNER_COUNT; i++)
-    {
-        XFreeCursor (display->dpy, display->resize_cursor[i]);
-        display->resize_cursor[i] = None;
-    }
-
     g_slist_free (display->clients);
     display->clients = NULL;
 
@@ -415,6 +380,52 @@ myDisplayHaveRender (DisplayInfo *display)
     return (display->have_render);
 }
 
+void
+myDisplayCreateCursor (DisplayInfo *display)
+{
+    display->root_cursor =
+        XCreateFontCursor (display->dpy, CURSOR_ROOT);
+    display->move_cursor =
+        XCreateFontCursor (display->dpy, CURSOR_MOVE);
+    display->busy_cursor =
+        cursorCreateSpinning (display->dpy);
+    display->resize_cursor[CORNER_TOP_LEFT] =
+        XCreateFontCursor (display->dpy, XC_top_left_corner);
+    display->resize_cursor[CORNER_TOP_RIGHT] =
+        XCreateFontCursor (display->dpy, XC_top_right_corner);
+    display->resize_cursor[CORNER_BOTTOM_LEFT] =
+        XCreateFontCursor (display->dpy, XC_bottom_left_corner);
+    display->resize_cursor[CORNER_BOTTOM_RIGHT] =
+        XCreateFontCursor (display->dpy, XC_bottom_right_corner);
+    display->resize_cursor[CORNER_COUNT + SIDE_LEFT] =
+        XCreateFontCursor (display->dpy, XC_left_side);
+    display->resize_cursor[CORNER_COUNT + SIDE_RIGHT] =
+        XCreateFontCursor (display->dpy, XC_right_side);
+    display->resize_cursor[CORNER_COUNT + SIDE_TOP] =
+        XCreateFontCursor (display->dpy, XC_top_side);
+    display->resize_cursor[CORNER_COUNT + SIDE_BOTTOM] =
+        XCreateFontCursor (display->dpy, XC_bottom_side);
+}
+
+void
+myDisplayFreeCursor (DisplayInfo *display)
+{
+    int i;
+
+    XFreeCursor (display->dpy, display->busy_cursor);
+    display->busy_cursor = None;
+    XFreeCursor (display->dpy, display->move_cursor);
+    display->move_cursor = None;
+    XFreeCursor (display->dpy, display->root_cursor);
+    display->root_cursor = None;
+
+    for (i = 0; i < SIDE_COUNT + CORNER_COUNT; i++)
+    {
+        XFreeCursor (display->dpy, display->resize_cursor[i]);
+        display->resize_cursor[i] = None;
+    }
+}
+
 Cursor
 myDisplayGetCursorBusy (DisplayInfo *display)
 {
@@ -607,17 +618,20 @@ myDisplayGetRootFromWindow(DisplayInfo *display, Window w)
 ScreenInfo *
 myDisplayGetScreenFromWindow (DisplayInfo *display, Window w)
 {
-    Window root;
+    GSList *list;
 
     g_return_val_if_fail (w != None, NULL);
     g_return_val_if_fail (display != NULL, NULL);
 
-    root = myDisplayGetRootFromWindow (display, w);
-    if (root != None)
+    for (list = display->screens; list; list = g_slist_next (list))
     {
-        return myDisplayGetScreenFromRoot (display, root);
+        ScreenInfo *screen = (ScreenInfo *) list->data;
+        if (screen->xroot == w)
+        {
+            return screen;
+        }
     }
-    TRACE ("myDisplayGetScreenFromWindow: no screen found");
+    TRACE ("myDisplayGetScreenFromWindow: no screen found for 0x%lx", w);
 
     return NULL;
 }
@@ -731,7 +745,7 @@ myDisplayUpdateCurrentTime (DisplayInfo *display, XEvent *ev)
             break;
     }
 
-    if ((timestamp == (guint32) CurrentTime) || TIMESTAMP_IS_BEFORE(display->current_time, timestamp))
+    if ((timestamp != (guint32) CurrentTime) && TIMESTAMP_IS_BEFORE(display->current_time, timestamp))
     {
         display->current_time = timestamp;
     }
