@@ -16,7 +16,7 @@
         MA 02110-1301, USA.
 
 
-        xfwm4    - (c) 2002-2011 Olivier Fourdan
+        xfwm4    - (c) 2002-2015 Olivier Fourdan
 
  */
 
@@ -109,8 +109,8 @@ enum
 {
     SIDE_LEFT = 0,
     SIDE_RIGHT,
-    SIDE_BOTTOM,
     SIDE_TOP,
+    SIDE_BOTTOM,
     SIDE_COUNT
 };
 #define NO_HANDLE -1
@@ -170,6 +170,8 @@ enum
 enum
 {
     COMPOSITING_MANAGER = 0,
+    GTK_FRAME_EXTENTS,
+    GTK_HIDE_TITLEBAR_WHEN_MAXIMIZED,
     KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR,
     KWM_WIN_ICON,
     MOTIF_WM_HINTS,
@@ -221,6 +223,7 @@ enum
     NET_WM_STATE_ABOVE,
     NET_WM_STATE_BELOW,
     NET_WM_STATE_DEMANDS_ATTENTION,
+    NET_WM_STATE_FOCUSED,
     NET_WM_STATE_FULLSCREEN,
     NET_WM_STATE_HIDDEN,
     NET_WM_STATE_MAXIMIZED_HORZ,
