@@ -17,7 +17,7 @@
 
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2011 Olivier Fourdan
+        xfwm4    - (c) 2002-2015 Olivier Fourdan
 
  */
 
@@ -82,8 +82,6 @@
 
 static GdkAtom atom_rcfiles = GDK_NONE;
 static xfwmWindow menu_event_window;
-static int edge_scroll_x = 0;
-static int edge_scroll_y = 0;
 
 /* Forward decl. */
 
@@ -413,7 +411,7 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
                 if ((guint) (key - KEY_MOVE_WORKSPACE_1) < screen_info->workspace_count)
                 {
                     clientRaise (c, None);
-                    workspaceSwitch (screen_info, key - KEY_MOVE_WORKSPACE_1, c, TRUE, ev->time);
+                    clientSetWorkspace (c, key - KEY_MOVE_WORKSPACE_1, TRUE);
                 }
                 break;
             case KEY_POPUP_MENU:
@@ -433,22 +431,42 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
             case KEY_TILE_DOWN:
                 clientTile (c, frameX (c) + frameWidth (c) / 2,
                                frameY (c) + frameHeight (c) / 2,
-                               TILE_DOWN, TRUE);
+                               TILE_DOWN, TRUE, TRUE);
                 break;
             case KEY_TILE_LEFT:
                 clientTile (c, frameX (c) + frameWidth (c) / 2,
                                frameY (c) + frameHeight (c) / 2,
-                               TILE_LEFT, TRUE);
+                               TILE_LEFT, TRUE, TRUE);
                 break;
             case KEY_TILE_RIGHT:
                 clientTile (c, frameX (c) + frameWidth (c) / 2,
                                frameY (c) + frameHeight (c) / 2,
-                               TILE_RIGHT, TRUE);
+                               TILE_RIGHT, TRUE, TRUE);
                 break;
             case KEY_TILE_UP:
                 clientTile (c, frameX (c) + frameWidth (c) / 2,
                                frameY (c) + frameHeight (c) / 2,
-                               TILE_UP, TRUE);
+                               TILE_UP, TRUE, TRUE);
+                break;
+            case KEY_TILE_DOWN_LEFT:
+                clientTile (c, frameX (c) + frameWidth (c) / 2,
+                               frameY (c) + frameHeight (c) / 2,
+                               TILE_DOWN_LEFT, TRUE, TRUE);
+                break;
+            case KEY_TILE_DOWN_RIGHT:
+                clientTile (c, frameX (c) + frameWidth (c) / 2,
+                               frameY (c) + frameHeight (c) / 2,
+                               TILE_DOWN_RIGHT, TRUE, TRUE);
+                break;
+            case KEY_TILE_UP_LEFT:
+                clientTile (c, frameX (c) + frameWidth (c) / 2,
+                               frameY (c) + frameHeight (c) / 2,
+                               TILE_UP_LEFT, TRUE, TRUE);
+                break;
+            case KEY_TILE_UP_RIGHT:
+                clientTile (c, frameX (c) + frameWidth (c) / 2,
+                               frameY (c) + frameHeight (c) / 2,
+                               TILE_UP_RIGHT, TRUE, TRUE);
                 break;
             default:
                 break;
@@ -801,11 +819,14 @@ titleButton (Client * c, guint state, XButtonEvent * ev)
     else if (ev->button == Button4)
     {
         /* Mouse wheel scroll up */
-        if (state == AltMask)
+#ifdef HAVE_COMPOSITOR
+        if ((state) && (state == screen_info->params->easy_click) && compositorIsActive (screen_info))
         {
             clientIncOpacity(c);
         }
-        else if (!FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
+        else
+#endif /* HAVE_COMPOSITOR */
+        if (!FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
             if (screen_info->params->mousewheel_rollup)
             {
@@ -816,11 +837,14 @@ titleButton (Client * c, guint state, XButtonEvent * ev)
     else if (ev->button == Button5)
     {
         /* Mouse wheel scroll down */
-        if (state == AltMask)
+#ifdef HAVE_COMPOSITOR
+        if ((state) && (state == screen_info->params->easy_click) && compositorIsActive (screen_info))
         {
             clientDecOpacity(c);
         }
-        else if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
+        else
+#endif /* HAVE_COMPOSITOR */
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
             if (screen_info->params->mousewheel_rollup)
             {
@@ -904,13 +928,13 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
             edgeButton (c, part, ev);
         }
 #ifdef HAVE_COMPOSITOR
-        else if ((ev->button == Button4) && (state) && (state == screen_info->params->easy_click)
-                  && (screen_info->compositor_active))
+        else if ((ev->button == Button4) && (screen_info->params->zoom_desktop) && (state) &&
+                 (state == screen_info->params->easy_click) && compositorIsActive (screen_info))
         {
             compositorZoomIn(screen_info, ev);
         }
-        else if ((ev->button == Button5) && (state) && (state == screen_info->params->easy_click)
-                  && (screen_info->compositor_active))
+        else if ((ev->button == Button5) && (screen_info->params->zoom_desktop) && (state) &&
+                 (state == screen_info->params->easy_click) && compositorIsActive (screen_info))
         {
             compositorZoomOut(screen_info, ev);
         }
@@ -1206,19 +1230,19 @@ handleUnmapNotify (DisplayInfo *display_info, XUnmapEvent * ev)
         return status;
     }
 
-    screen_info = myDisplayGetScreenFromWindow (display_info, ev->window);
-    if (screen_info && (ev->event != ev->window) && (ev->event != screen_info->xroot || !ev->send_event))
-    {
-        TRACE ("handleUnmapNotify (): Event ignored");
-        return status;
-    }
-
     c = myDisplayGetClientFromWindow (display_info, ev->window, SEARCH_WINDOW);
     if (c)
     {
         TRACE ("UnmapNotify for \"%s\" (0x%lx)", c->name, c->window);
         TRACE ("ignore_unmap for \"%s\" is %i", c->name, c->ignore_unmap);
 
+        screen_info = c->screen_info;
+        if ((ev->event != ev->window) && (ev->event != screen_info->xroot || !ev->send_event))
+        {
+            TRACE ("handleUnmapNotify (): Event ignored");
+            return status;
+        }
+
         status = EVENT_FILTER_REMOVE;
         if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MAP_PENDING))
         {
@@ -1231,8 +1255,6 @@ handleUnmapNotify (DisplayInfo *display_info, XUnmapEvent * ev)
             return status;
         }
 
-        screen_info = c->screen_info;
-
         /*
          * ICCCM spec states that a client wishing to switch
          * to WithdrawnState should send a synthetic UnmapNotify
@@ -1335,8 +1357,8 @@ handleConfigureRequest (DisplayInfo *display_info, XConfigureRequestEvent * ev)
             /* Sorry, but it's not the right time for configure request */
             return EVENT_FILTER_REMOVE;
         }
-        clientAdjustCoordGravity (c, c->gravity, &ev->value_mask, &wc);
         clientMoveResizeWindow (c, &wc, ev->value_mask);
+        clientAdjustCoordGravity (c, c->gravity, &ev->value_mask, &wc);
     }
     else
     {
@@ -1350,11 +1372,10 @@ handleConfigureRequest (DisplayInfo *display_info, XConfigureRequestEvent * ev)
 static eventFilterStatus
 handleEnterNotify (DisplayInfo *display_info, XCrossingEvent * ev)
 {
-    static guint32 lastresist = CurrentTime;
     ScreenInfo *screen_info;
     Client *c;
     int b;
-    gboolean warp_pointer, need_redraw;
+    gboolean need_redraw;
 
     /* See http://rfc-ref.org/RFC-TEXTS/1013/chapter12.html for details */
 
@@ -1369,7 +1390,6 @@ handleEnterNotify (DisplayInfo *display_info, XCrossingEvent * ev)
 
     TRACE ("EnterNotify on window (0x%lx)", ev->window);
 
-    warp_pointer = FALSE;
     need_redraw = FALSE;
     c = myDisplayGetClientFromWindow (display_info, ev->window, SEARCH_FRAME | SEARCH_BUTTON);
     if (c)
@@ -1435,107 +1455,9 @@ handleEnterNotify (DisplayInfo *display_info, XCrossingEvent * ev)
         return EVENT_FILTER_PASS;
     }
 
-    if (screen_info->workspace_count && screen_info->params->wrap_workspaces
-        && screen_info->params->wrap_resistance)
+    if (screen_info->params->wrap_workspaces && screen_info->workspace_count > 1)
     {
-        int msx, msy, maxx, maxy;
-        int rx, ry, delta;
-
-        msx = ev->x_root;
-        msy = ev->y_root;
-        maxx = screen_info->width - 1;
-        maxy = screen_info->height - 1;
-        rx = 0;
-        ry = 0;
-        warp_pointer = FALSE;
-
-        if ((msx == 0) || (msx == maxx))
-        {
-            if ((ev->time - lastresist) > 250)  /* ms */
-            {
-                edge_scroll_x = 0;
-            }
-            else
-            {
-                edge_scroll_x++;
-            }
-            if (msx == 0)
-            {
-                rx = 1;
-            }
-            else
-            {
-                rx = -1;
-            }
-            warp_pointer = TRUE;
-            lastresist = ev->time;
-        }
-        if ((msy == 0) || (msy == maxy))
-        {
-            if ((ev->time - lastresist) > 250)  /* ms */
-            {
-                edge_scroll_y = 0;
-            }
-            else
-            {
-                edge_scroll_y++;
-            }
-            if (msy == 0)
-            {
-                ry = 1;
-            }
-            else
-            {
-                ry = -1;
-            }
-            warp_pointer = TRUE;
-            lastresist = ev->time;
-        }
-
-        if (edge_scroll_x > screen_info->params->wrap_resistance)
-        {
-            edge_scroll_x = 0;
-            delta = MAX (9 * maxx / 10, maxx - 5 * screen_info->params->wrap_resistance);
-            if (msx == 0)
-            {
-                if (workspaceMove (screen_info, 0, -1, NULL, ev->time))
-                {
-                    rx = delta;
-                }
-            }
-            else
-            {
-                if (workspaceMove (screen_info, 0, 1, NULL, ev->time))
-                {
-                    rx = -delta;
-                }
-            }
-            warp_pointer = TRUE;
-        }
-        if (edge_scroll_y > screen_info->params->wrap_resistance)
-        {
-            edge_scroll_y = 0;
-            delta = MAX (9 * maxy / 10, maxy - 5 * screen_info->params->wrap_resistance);
-            if (msy == 0)
-            {
-                if (workspaceMove (screen_info, -1, 0, NULL, ev->time))
-                {
-                    ry = delta;
-                }
-            }
-            else
-            {
-                if (workspaceMove (screen_info, 1, 0, NULL, ev->time))
-                {
-                    ry = -delta;
-                }
-            }
-            warp_pointer = TRUE;
-        }
-        if (warp_pointer)
-        {
-            XWarpPointer (display_info->dpy, None, None, 0, 0, 0, 0, rx, ry);
-        }
+        clientMoveWarp (NULL, screen_info, &ev->x_root, &ev->y_root, ev->time);
     }
 
     return EVENT_FILTER_REMOVE;
@@ -1618,16 +1540,16 @@ handleFocusIn (DisplayInfo *display_info, XFocusChangeEvent * ev)
                 "NotifyDetailNone" :
                 "(unknown)");
 
-    screen_info = myDisplayGetScreenFromWindow (display_info, ev->window);
-    if (!screen_info)
+    if ((ev->mode == NotifyGrab) || (ev->mode == NotifyUngrab))
     {
-        /* Not for us */
+        /* We're not interested in such notifications */
         return EVENT_FILTER_PASS;
     }
 
-    if ((ev->window == screen_info->xroot)
-        && ((ev->detail == NotifyDetailNone)
-            || ((ev->mode == NotifyNormal) && (ev->detail == NotifyInferior))))
+    screen_info = myDisplayGetScreenFromWindow (display_info, ev->window);
+    if (screen_info &&
+        ((ev->detail == NotifyDetailNone) ||
+         ((ev->mode == NotifyNormal) && (ev->detail == NotifyInferior))))
     {
         /*
            Handle unexpected focus transition to root (means that an unknown
@@ -1637,11 +1559,7 @@ handleFocusIn (DisplayInfo *display_info, XFocusChangeEvent * ev)
         clientSetFocus (screen_info, c, getXServerTime (display_info), FOCUS_FORCE);
         return EVENT_FILTER_PASS;
     }
-    if ((ev->mode == NotifyGrab) || (ev->mode == NotifyUngrab))
-    {
-        /* We're not interested in such notifications */
-        return EVENT_FILTER_PASS;
-    }
+
     c = myDisplayGetClientFromWindow (display_info, ev->window, SEARCH_FRAME | SEARCH_WINDOW);
     user_focus = clientGetUserFocus ();
     current_focus = clientGetFocus ();
@@ -1880,16 +1798,16 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
         }
         else if (ev->atom == display_info->atoms[NET_WM_WINDOW_OPACITY])
         {
-            TRACE ("client \"%s\" (0x%lx) has received a NET_WM_OPACITY notify", c->name, c->window);
+            TRACE ("client \"%s\" (0x%lx) has received a NET_WM_WINDOW_OPACITY notify", c->name, c->window);
             if (!getOpacity (display_info, c->window, &c->opacity))
             {
                 c->opacity =  NET_WM_OPAQUE;
             }
-            compositorWindowSetOpacity (display_info, c->frame, c->opacity);
+            clientSetOpacity (c, c->opacity, 0, 0);
         }
         else if (ev->atom == display_info->atoms[NET_WM_WINDOW_OPACITY_LOCKED])
         {
-            TRACE ("client \"%s\" (0x%lx) has received a NET_WM_OPACITY_LOCKED notify", c->name, c->window);
+            TRACE ("client \"%s\" (0x%lx) has received a NET_WM_WINDOW_OPACITY_LOCKED notify", c->name, c->window);
             if (getOpacityLock (display_info, c->window))
             {
                 FLAG_SET (c->xfwm_flags, XFWM_FLAG_OPACITY_LOCKED);
@@ -1905,6 +1823,22 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
         {
             clientUpdateIcon (c);
         }
+        else if (ev->atom == display_info->atoms[GTK_FRAME_EXTENTS])
+        {
+            TRACE ("client \"%s\" (0x%lx) has received a GTK_FRAME_EXTENTS notify", c->name, c->window);
+            clientGetGtkFrameExtents (c);
+            clientUpdateMaximizeSize (c);
+        }
+        else if (ev->atom == display_info->atoms[GTK_HIDE_TITLEBAR_WHEN_MAXIMIZED])
+        {
+            TRACE ("client \"%s\" (0x%lx) has received a GTK_HIDE_TITLEBAR_WHEN_MAXIMIZED notify", c->name, c->window);
+            clientGetGtkHideTitlebar (c);
+            if (FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED))
+            {
+                clientUpdateMaximizeSize (c);
+                clientReconfigure (c, CFG_FORCE_REDRAW);
+            }
+        }
 #ifdef HAVE_STARTUP_NOTIFICATION
         else if (ev->atom == display_info->atoms[NET_STARTUP_ID])
         {
@@ -1919,8 +1853,8 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
 #ifdef HAVE_XSYNC
         else if (ev->atom == display_info->atoms[NET_WM_SYNC_REQUEST_COUNTER])
         {
-            getXSyncCounter (display_info, c->window, &c->xsync_counter);
-            TRACE ("Window 0x%lx has NET_WM_SYNC_REQUEST_COUNTER set to 0x%lx", c->window, c->xsync_counter);
+            TRACE ("Window 0x%lx has received NET_WM_SYNC_REQUEST_COUNTER", c->window);
+            clientGetXSyncCounter (c);
         }
 #endif /* HAVE_XSYNC */
 
@@ -2078,7 +2012,6 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
 
             TRACE ("window (0x%lx) has received a MANAGER event", ev->window);
             selection = (Atom) ev->data.l[1];
-
 #ifdef ENABLE_KDE_SYSTRAY_PROXY
             if (selection == screen_info->net_system_tray_selection)
             {
@@ -2114,12 +2047,24 @@ static eventFilterStatus
 handleSelectionClear (DisplayInfo *display_info, XSelectionClearEvent * ev)
 {
     eventFilterStatus status;
-    ScreenInfo *screen_info;
+    ScreenInfo *screen_info, *pscreen;
+    GSList *list;
 
-    TRACE ("entering handleSelectionClear");
+    DBG ("entering handleSelectionClear 0x%lx", ev->window);
 
     status = EVENT_FILTER_PASS;
-    screen_info = myDisplayGetScreenFromWindow (display_info, ev->window);
+    screen_info = NULL;
+
+    for (list = display_info->screens; list; list = g_slist_next (list))
+    {
+        pscreen = (ScreenInfo *) list->data;
+        if (ev->window == pscreen->xfwm4_win)
+        {
+            screen_info = pscreen;
+            break;
+        }
+    }
+
     if (screen_info)
     {
         if (myScreenCheckWMAtom (screen_info, ev->selection))
@@ -2208,20 +2153,14 @@ handleXSyncAlarmNotify (DisplayInfo *display_info, XSyncAlarmNotifyEvent * ev)
     Client *c;
 
     TRACE ("entering handleXSyncAlarmNotify");
-
-    if (!display_info->have_xsync)
+    if (display_info->have_xsync)
     {
-        return EVENT_FILTER_REMOVE;
+        c = myDisplayGetClientFromXSyncAlarm (display_info, ev->alarm);
+        if (c)
+        {
+            clientXSyncUpdateValue (c, ev->counter_value);
+        }
     }
-
-    c = myDisplayGetClientFromXSyncAlarm (display_info, ev->alarm);
-    if (c)
-    {
-        c->xsync_waiting = FALSE;
-        c->xsync_value = ev->counter_value;
-        clientXSyncClearTimeout (c);
-    }
-
     return EVENT_FILTER_REMOVE;
 }
 #endif /* HAVE_XSYNC */
@@ -2811,6 +2750,8 @@ size_changed_cb(GdkScreen *gscreen, gpointer data)
 
         setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count,
                         screen_info->width, screen_info->height, screen_info->margins);
+        setNetDesktopInfo (display_info, screen_info->xroot, screen_info->current_ws,
+                           screen_info->width, screen_info->height);
 
         placeSidewalks (screen_info, screen_info->params->wrap_workspaces);
 
@@ -2864,6 +2805,8 @@ monitors_changed_cb(GdkScreen *gscreen, gpointer data)
 
         setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count,
                         screen_info->width, screen_info->height, screen_info->margins);
+        setNetDesktopInfo (display_info, screen_info->xroot, screen_info->current_ws,
+                           screen_info->width, screen_info->height);
 
         placeSidewalks (screen_info, screen_info->params->wrap_workspaces);
     }
