@@ -18,7 +18,7 @@
 
         oroborus - (c) 2001 Ken Lynch
         Metacity - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2011 Olivier Fourdan
+        xfwm4    - (c) 2002-2015 Olivier Fourdan
 
  */
 
@@ -242,7 +242,7 @@ getHint (DisplayInfo *display_info, Window w, int atom_id, long *value)
                              FALSE, XA_CARDINAL, &real_type, &real_format, &items_read, &items_left,
                              (unsigned char **) &data) == Success) && (items_read))
     {
-        *value = *((long *) data);
+        *value = *((long *) data) & ((1LL << real_format) - 1);
         if (data)
         {
             XFree (data);
@@ -337,52 +337,6 @@ getDesktopLayout (DisplayInfo *display_info, Window root, int ws_count, NetWmDes
     }
 }
 
-void
-getGnomeDesktopMargins (DisplayInfo *display_info, Window root, int * m)
-{
-    Atom real_type;
-    int real_format;
-    unsigned long items_read, items_left;
-    unsigned long *ptr;
-    unsigned char *data;
-
-    TRACE ("entering getGnomeDesktopMargins");
-
-    ptr = NULL;
-    data = NULL;
-    if ((XGetWindowProperty (display_info->dpy, root,
-                display_info->atoms[GNOME_PANEL_DESKTOP_AREA], 0L, 4L, FALSE, XA_CARDINAL,
-                &real_type, &real_format, &items_read, &items_left,
-                (unsigned char **) &data) == Success) && (items_read >= 4))
-    {
-        ptr = (unsigned long *) data;
-        m[0] = (int) *ptr++;
-        m[1] = (int) *ptr++;
-        m[2] = (int) *ptr++;
-        m[3] = (int) *ptr++;
-        XFree (data);
-    }
-    else
-    {
-        m[0] = 0;
-        m[1] = 0;
-        m[2] = 0;
-        m[3] = 0;
-    }
-}
-
-void
-setGnomeProtocols (DisplayInfo *display_info, Window root, Window w)
-{
-    Atom atoms[1];
-
-    atoms[0] = display_info->atoms[WIN_LAYER];
-    XChangeProperty (display_info->dpy, root, display_info->atoms[WIN_PROTOCOLS], XA_ATOM,
-                     32, PropModeReplace, (unsigned char *) atoms, 1);
-    setHint (display_info, w, WIN_SUPPORTING_WM_CHECK, w);
-    setHint (display_info, root, WIN_SUPPORTING_WM_CHECK, w);
-}
-
 void
 setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
 {
@@ -435,6 +389,7 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
     atoms[i++] = display_info->atoms[NET_WM_STATE_ABOVE];
     atoms[i++] = display_info->atoms[NET_WM_STATE_BELOW];
     atoms[i++] = display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION];
+    atoms[i++] = display_info->atoms[NET_WM_STATE_FOCUSED];
     atoms[i++] = display_info->atoms[NET_WM_STATE_FULLSCREEN];
     atoms[i++] = display_info->atoms[NET_WM_STATE_HIDDEN];
     atoms[i++] = display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ];
@@ -523,6 +478,7 @@ getCardinalList (DisplayInfo *display_info, Window w, int atom_id, unsigned long
 {
     Atom type;
     int format;
+    unsigned int i;
     unsigned long n_cardinals;
     unsigned long bytes_after;
     unsigned char *data;
@@ -550,6 +506,10 @@ getCardinalList (DisplayInfo *display_info, Window w, int atom_id, unsigned long
 
     *cardinals_p = cardinals;
     *n_cardinals_p = n_cardinals;
+    for (i = 0; i < n_cardinals; i++)
+    {
+        (*cardinals_p)[i] = (*cardinals_p)[i] & ((1LL << format) - 1);
+    }
 
     return TRUE;
 }
@@ -604,15 +564,22 @@ setNetFullscreenMonitors (DisplayInfo *display_info, Window w, gint top, gint bo
                      XA_CARDINAL, 32, PropModeReplace, (unsigned char *) data, 4);
 }
 
+int
+getNetCurrentDesktop (DisplayInfo *display_info, Window root)
+{
+    long ws;
+
+    TRACE ("entering getNetCurrentDesktop");
+    getHint (display_info, root, NET_CURRENT_DESKTOP, &ws);
+    return (int) ws;
+}
+
 void
-initNetDesktopInfo (DisplayInfo *display_info, Window root, int workspace, int width, int height)
+setNetCurrentDesktop (DisplayInfo *display_info, Window root, int workspace)
 {
     unsigned long data[2];
-    TRACE ("entering initNetDesktopInfo");
-    data[0] = width;
-    data[1] = height;
-    XChangeProperty (display_info->dpy, root, display_info->atoms[NET_DESKTOP_GEOMETRY],
-                     XA_CARDINAL, 32, PropModeReplace, (unsigned char *) data, 2);
+    TRACE ("entering setNetCurrentDesktop");
+
     data[0] = 0;
     data[1] = 0;
     XChangeProperty (display_info->dpy, root, display_info->atoms[NET_DESKTOP_VIEWPORT],
@@ -622,6 +589,18 @@ initNetDesktopInfo (DisplayInfo *display_info, Window root, int workspace, int w
                      XA_CARDINAL, 32, PropModeReplace, (unsigned char *) data, 1);
 }
 
+void
+setNetDesktopInfo (DisplayInfo *display_info, Window root, int workspace, int width, int height)
+{
+    unsigned long data[2];
+    TRACE ("entering setNetDesktopInfo");
+    data[0] = width;
+    data[1] = height;
+    XChangeProperty (display_info->dpy, root, display_info->atoms[NET_DESKTOP_GEOMETRY],
+                     XA_CARDINAL, 32, PropModeReplace, (unsigned char *) data, 2);
+    setNetCurrentDesktop (display_info, root, workspace);
+}
+
 void
 setUTF8StringHint (DisplayInfo *display_info, Window w, int atom_id, const gchar *val)
 {
@@ -1140,7 +1119,7 @@ getRGBIconData (DisplayInfo *display_info, Window window, unsigned long **data,
 }
 
 gboolean
-getOpacity (DisplayInfo *display_info, Window window, guint *opacity)
+getOpacity (DisplayInfo *display_info, Window window, guint32 *opacity)
 {
     long val;
 
@@ -1151,7 +1130,7 @@ getOpacity (DisplayInfo *display_info, Window window, guint *opacity)
     val = 0;
     if (getHint (display_info, window, NET_WM_WINDOW_OPACITY, &val))
     {
-        *opacity = (guint) val;
+        *opacity = (guint32) val;
         return TRUE;
     }
 
