@@ -246,7 +246,6 @@ workspaceSwitch (ScreenInfo *screen_info, gint new_ws, Client * c2, gboolean upd
     Window dr, window;
     gint rx, ry, wx, wy;
     unsigned int mask;
-    unsigned long data[1];
 
     g_return_if_fail (screen_info != NULL);
 
@@ -368,11 +367,7 @@ workspaceSwitch (ScreenInfo *screen_info, gint new_ws, Client * c2, gboolean upd
         }
     }
 
-    setHint (display_info, screen_info->xroot, WIN_WORKSPACE, new_ws);
-    data[0] = new_ws;
-    XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot,
-                     display_info->atoms[NET_CURRENT_DESKTOP], XA_CARDINAL, 32,
-                     PropModeReplace, (unsigned char *) data, 1);
+    setNetCurrentDesktop (display_info, screen_info->xroot, new_ws);
     if (!(screen_info->params->click_to_focus))
     {
         if (!(c2) && (XQueryPointer (myScreenGetXDisplay (screen_info), screen_info->xroot, &dr, &window, &rx, &ry, &wx, &wy, &mask)))
@@ -427,7 +422,7 @@ workspaceSetCount (ScreenInfo * screen_info, guint count)
 {
     DisplayInfo *display_info;
     Client *c;
-    guint i;
+    GList *list;
 
     g_return_if_fail (screen_info != NULL);
 
@@ -443,12 +438,12 @@ workspaceSetCount (ScreenInfo * screen_info, guint count)
     }
 
     display_info = screen_info->display_info;
-    setHint (display_info, screen_info->xroot, WIN_WORKSPACE_COUNT, count);
     setHint (display_info, screen_info->xroot, NET_NUMBER_OF_DESKTOPS, count);
     screen_info->workspace_count = count;
 
-    for (c = screen_info->clients, i = 0; i < screen_info->client_count; c = c->next, i++)
+    for (list = screen_info->windows_stack; list; list = g_list_next (list))
     {
+        c = (Client *) list->data;
         if (c->win_workspace > count - 1)
         {
             clientSetWorkspace (c, count - 1, TRUE);
@@ -469,7 +464,8 @@ void
 workspaceInsert (ScreenInfo * screen_info, guint position)
 {
     Client *c;
-    guint i, count;
+    GList *list;
+    guint count;
 
     g_return_if_fail (screen_info != NULL);
 
@@ -483,8 +479,9 @@ workspaceInsert (ScreenInfo * screen_info, guint position)
         return;
     }
 
-    for (c = screen_info->clients, i = 0; i < screen_info->client_count; c = c->next, i++)
+    for (list = screen_info->windows_stack; list; list = g_list_next (list))
     {
+        c = (Client *) list->data;
         if (c->win_workspace >= position)
         {
             clientSetWorkspace (c, c->win_workspace + 1, TRUE);
@@ -530,6 +527,8 @@ workspaceUpdateArea (ScreenInfo *screen_info)
     int prev_bottom;
     guint i;
 
+    GdkRectangle workarea,struttmp;
+
     g_return_if_fail (screen_info != NULL);
     g_return_if_fail (screen_info->margins != NULL);
     g_return_if_fail (screen_info->gnome_margins != NULL);
@@ -547,11 +546,65 @@ workspaceUpdateArea (ScreenInfo *screen_info)
         screen_info->margins[i] = screen_info->gnome_margins[i];
     }
 
+    gdk_screen_get_monitor_geometry (screen_info->gscr, gdk_screen_get_primary_monitor(screen_info->gscr), &workarea);
+
     for (c = screen_info->clients, i = 0; i < screen_info->client_count; c = c->next, i++)
     {
         if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STRUT)
             && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
         {
+            /*
+             * NET_WORKAREA doesn't support L shaped displays at all.
+             * gdk works around this by ignoring it unless dealing with
+             * the primary monitor.
+             * Mimic this behaviour by ignoring struts not on the primary
+             * display when calculating NET_WORKAREA
+             */
+
+            if(c->struts[STRUTS_TOP])
+            {
+                struttmp.x = c->struts[STRUTS_TOP_START_X];
+                struttmp.y = 0;
+                struttmp.width = c->struts[STRUTS_TOP_END_X] - c->struts[STRUTS_TOP_START_X];
+                struttmp.height = c->struts[STRUTS_TOP];
+
+                if (!gdk_rectangle_intersect (&workarea, &struttmp, NULL))
+                    continue;
+            }
+
+            if(c->struts[STRUTS_BOTTOM])
+            {
+                struttmp.x = c->struts[STRUTS_BOTTOM_START_X];
+                struttmp.y = screen_info->height - c->struts[STRUTS_BOTTOM];
+                struttmp.width = c->struts[STRUTS_BOTTOM_END_X] - c->struts[STRUTS_BOTTOM_START_X];
+                struttmp.height = c->struts[STRUTS_BOTTOM];
+
+                if (!gdk_rectangle_intersect (&workarea, &struttmp, NULL))
+                    continue;
+            }
+
+            if(c->struts[STRUTS_LEFT])
+            {
+                struttmp.x = 0;
+                struttmp.y = c->struts[STRUTS_LEFT_START_Y];
+                struttmp.width = c->struts[STRUTS_LEFT];
+                struttmp.height = c->struts[STRUTS_LEFT_END_Y] - c->struts[STRUTS_LEFT_START_Y];
+
+                if (!gdk_rectangle_intersect (&workarea, &struttmp, NULL))
+                    continue;
+            }
+
+            if(c->struts[STRUTS_RIGHT])
+            {
+                struttmp.x = screen_info->width - c->struts[STRUTS_RIGHT];
+                struttmp.y = c->struts[STRUTS_RIGHT_START_Y];
+                struttmp.width = c->struts[STRUTS_RIGHT];
+                struttmp.height = c->struts[STRUTS_RIGHT_END_Y] - c->struts[STRUTS_RIGHT_START_Y];
+
+                if (!gdk_rectangle_intersect (&workarea, &struttmp, NULL))
+                    continue;
+            }
+
             screen_info->margins[STRUTS_TOP]    = MAX (screen_info->margins[STRUTS_TOP],    c->struts[STRUTS_TOP]);
             screen_info->margins[STRUTS_LEFT]   = MAX (screen_info->margins[STRUTS_LEFT],   c->struts[STRUTS_LEFT]);
             screen_info->margins[STRUTS_RIGHT]  = MAX (screen_info->margins[STRUTS_RIGHT],  c->struts[STRUTS_RIGHT]);
